import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { PerformanceIndicator } from 'models'
import React, { useEffect, useState } from 'react'
import { useSelector } from 'react-redux'
import ConsumptionService from 'services/consumption.service'
import ConfigService from 'services/fluidConfig.service'
import PerformanceIndicatorService from 'services/performanceIndicator.service'
import { AppStore } from 'store'
import AnalysisConsumption from './AnalysisConsumption'
import AnalysisErrorModal from './AnalysisErrorModal'
import ComparisonView from './ComparisonView'
import ElecHalfHourMonthlyAnalysis from './ElecHalfHourMonthlyAnalysis'
import MaxConsumptionCard from './MaxConsumptionCard'
import TotalAnalysisChart from './TotalAnalysisChart'
import './monthlyanalysis.scss'

interface MonthlyAnalysisProps {
  saveLastScrollPosition: () => void
  scrollPosition: number
}

const MonthlyAnalysis = ({
  saveLastScrollPosition,
  scrollPosition,
}: MonthlyAnalysisProps) => {
  const client = useClient()
  const {
    analysis: { analysisMonth },
    global: { fluidTypes },
  } = useSelector((state: AppStore) => state.ecolyo)
  const [performanceIndicators, setPerformanceIndicators] = useState<
    PerformanceIndicator[]
  >([])
  const [aggregatedPerformanceIndicators, setAggregatedPerformanceIndicators] =
    useState<PerformanceIndicator>({
      value: 0,
      compareValue: 0,
      percentageVariation: 0,
    })
  const [loadAnalysis, setLoadAnalysis] = useState<boolean>(true)
  const configService = new ConfigService()
  const fluidConfig = configService.getFluidConfig()

  useEffect(() => {
    let subscribed = true
    async function populateData() {
      setLoadAnalysis(true)
      const consumptionService = new ConsumptionService(client)
      const performanceIndicatorService = new PerformanceIndicatorService()
      const periods = {
        timePeriod: {
          startDate: analysisMonth.minus({ month: 1 }).startOf('month'),
          endDate: analysisMonth.minus({ month: 1 }).endOf('month'),
        },
        comparisonTimePeriod: {
          startDate: analysisMonth.minus({ month: 2 }).startOf('month'),
          endDate: analysisMonth.minus({ month: 2 }).endOf('month'),
        },
      }
      const fetchedPerformanceIndicators =
        await consumptionService.getPerformanceIndicators(
          periods.timePeriod,
          TimeStep.MONTH,
          fluidTypes,
          periods.comparisonTimePeriod
        )
      if (subscribed) {
        if (fetchedPerformanceIndicators) {
          setPerformanceIndicators(fetchedPerformanceIndicators)
          setLoadAnalysis(false)
          const hasValidPI = fetchedPerformanceIndicators.some(pi => pi?.value)
          if (hasValidPI) setLoadAnalysis(true)

          setAggregatedPerformanceIndicators(
            performanceIndicatorService.aggregatePerformanceIndicators(
              fetchedPerformanceIndicators
            )
          )
        }
        setLoadAnalysis(false)
      }
    }
    populateData()

    return () => {
      saveLastScrollPosition()
      subscribed = false
    }
  }, [client, fluidTypes, analysisMonth, saveLastScrollPosition])

  useEffect(() => {
    if (!loadAnalysis) {
      const app = document.querySelector('.app-content')
      window.scrollTo(0, scrollPosition)
      app?.scrollTo(0, scrollPosition)
    }
  }, [loadAnalysis, scrollPosition])

  return (
    <>
      {loadAnalysis && (
        <div className="analysis-container-spinner" aria-busy="true">
          <Loader />
        </div>
      )}
      {!loadAnalysis && (
        <div className="analysis-root black">
          {fluidTypes.length >= 1 ? (
            <>
              <div className="analysis-content">
                <ComparisonView fluidConfig={fluidConfig} />
              </div>
              <div className="analysis-content">
                <div className="card rich-card">
                  <TotalAnalysisChart fluidTypes={fluidTypes} />
                </div>
              </div>
              <div className="analysis-content">
                <div className="card rich-card">
                  <MaxConsumptionCard />
                </div>
              </div>
              <div className="analysis-content">
                <div className="card rich-card">
                  <AnalysisConsumption
                    aggregatedPerformanceIndicator={
                      aggregatedPerformanceIndicators
                    }
                    performanceIndicators={performanceIndicators}
                  />
                </div>
              </div>
              {fluidTypes.includes(FluidType.ELECTRICITY) && (
                <div className="analysis-content">
                  <div className="card">
                    <ElecHalfHourMonthlyAnalysis
                      perfIndicator={
                        performanceIndicators[FluidType.ELECTRICITY]
                      }
                    />
                  </div>
                </div>
              )}
            </>
          ) : (
            <AnalysisErrorModal />
          )}
        </div>
      )}
    </>
  )
}

export default MonthlyAnalysis
