import { Button, Dialog, IconButton } from '@material-ui/core'
import CloseIcon from 'assets/icons/ico/close.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import FormOAuth from 'components/Connection/FormOAuth'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidStatus, Konnector } from 'models'
import { Step } from 'models/step.model'
import React, { useCallback, useEffect, useState } from 'react'
import './PartnerConnectModal.scss'
import GrdfCreateAccount from './Steps/GrdfCreateAccount'
import GrdfDoYouHaveAccount from './Steps/GrdfDoYouHaveAccount'
import GiveConsent from './Steps/GrdfGiveConsent'

enum StepEnum {
  DoYouHaveAccount,
  CreateAccount,
  GiveConsent,
}

interface GrdfConnectModalProps {
  open: boolean
  showForm: boolean
  konnector: Konnector | null
  fluidStatus: FluidStatus
  handleCloseClick: () => void
  setShowForm: (value: boolean) => void
  goToPartnerSite: () => void
  handleSuccess: (accountId: string) => Promise<void>
}

const GrdfConnectModal = ({
  open,
  showForm,
  konnector,
  fluidStatus,
  handleCloseClick,
  setShowForm,
  goToPartnerSite,
  handleSuccess,
}: GrdfConnectModalProps) => {
  const { t } = useI18n()
  const [currentStep, setCurrentStep] = useState(StepEnum.DoYouHaveAccount)
  useEffect(() => {
    if (open) {
      showForm
        ? setCurrentStep(StepEnum.GiveConsent)
        : setCurrentStep(StepEnum.DoYouHaveAccount)
    }
  }, [showForm, open])

  const goToCreateAccountStep = useCallback(() => {
    setCurrentStep(StepEnum.CreateAccount)
  }, [])
  const goToGiveConsentStep = useCallback(() => {
    setCurrentStep(StepEnum.GiveConsent)
  }, [])
  const goToDoYouHaveAccountStep = useCallback(() => {
    setCurrentStep(StepEnum.DoYouHaveAccount)
  }, [])
  const handleGoToPartnerSite = useCallback(() => {
    goToPartnerSite()
    setShowForm(true)
    handleCloseClick()
  }, [goToPartnerSite, handleCloseClick, setShowForm])
  const handleGiveConsentPrevious = useCallback(
    () =>
      showForm ? handleCloseClick() : setCurrentStep(StepEnum.DoYouHaveAccount),
    [handleCloseClick, showForm]
  )

  const steps: Record<StepEnum, Step> = {
    [StepEnum.DoYouHaveAccount]: {
      content: <GrdfDoYouHaveAccount />,
      leftButton: (
        <Button
          onClick={goToCreateAccountStep}
          classes={{
            root: 'btn-profile-back',
            label: 'text-16-bold',
          }}
        >
          {t('auth.button_create_account')}
        </Button>
      ),
      rightButton: (
        <Button
          onClick={goToGiveConsentStep}
          classes={{
            root: 'btn-profile-next',
            label: 'text-16-bold',
          }}
        >
          {t('auth.button_has_account')}
        </Button>
      ),
    },
    [StepEnum.CreateAccount]: {
      content: <GrdfCreateAccount />,
      leftButton: (
        <Button
          onClick={goToDoYouHaveAccountStep}
          classes={{
            root: 'btn-profile-back',
            label: 'text-16-bold',
          }}
        >
          {`< ${t('auth.button_previous')}`}
        </Button>
      ),
      rightButton: (
        <Button
          onClick={handleGoToPartnerSite}
          classes={{
            root: 'btn-profile-next',
            label: 'text-16-bold',
          }}
        >
          {t('auth.grdfgrandlyon.button_go_to_partner_site')}
        </Button>
      ),
    },
    [StepEnum.GiveConsent]: {
      content: <GiveConsent />,
      leftButton: (
        <Button
          onClick={handleGiveConsentPrevious}
          classes={{
            root: 'btn-profile-back',
            label: 'text-16-bold',
          }}
        >
          {`< ${t('auth.button_previous')}`}
        </Button>
      ),
      rightButton: (
        <FormOAuth
          konnector={konnector}
          onSuccess={handleSuccess}
          fluidStatus={fluidStatus}
        />
      ),
    },
  }

  return (
    <Dialog
      open={open}
      onClose={handleCloseClick}
      classes={{
        root: 'modal-root',
        paper: 'modal-paper',
      }}
    >
      <IconButton
        aria-label={t('auth.accessibility.button_close')}
        className="modal-paper-close-button"
        onClick={handleCloseClick}
      >
        <StyledIcon icon={CloseIcon} size={16} />
      </IconButton>
      <div className="partners-connection-step-content">
        {steps[currentStep].content}
      </div>
      <div className="partners-connection-step-navigation">
        {steps[currentStep].leftButton}
        {steps[currentStep].rightButton}
      </div>
    </Dialog>
  )
}

export default GrdfConnectModal
