import { ListItemIcon, Menu, MenuItem } from '@material-ui/core/'
import Button from '@material-ui/core/Button'
import CheckIcon from 'assets/icons/ico/check.svg'
import SortIcon from 'assets/icons/ico/sort.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import EcogestureCard from 'components/Ecogesture/EcogestureCard'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { Usage } from 'enum/ecogesture.enum'
import { Ecogesture } from 'models'
import React, { useState } from 'react'
import { useNavigate } from 'react-router-dom'
import './ecogestureList.scss'

interface EcogestureListProps {
  list: Ecogesture[]
  displaySelection: boolean
  selectionTotal: number
  selectionViewed: number
  handleReinitClick?: () => void
}

const EcogestureList = ({
  list,
  displaySelection,
  selectionTotal,
  selectionViewed,
  handleReinitClick,
}: EcogestureListProps) => {
  const { t } = useI18n()
  const navigate = useNavigate()
  const [activeFilter, setActiveFilter] = useState<string>(Usage[Usage.ALL])
  const [openDropDown, setOpenDropDown] = useState<boolean>(false)
  const [anchorEl, setAnchorEl] = useState<null | HTMLElement>(null)

  const toggleDropDown = () => {
    setOpenDropDown(prev => !prev)
  }

  const toggleMenu = (event: React.MouseEvent<HTMLButtonElement>) => {
    setAnchorEl(event.currentTarget)
  }

  const closeMenu = (usage?: string) => {
    usage && setActiveFilter(usage)
    setAnchorEl(null)
  }

  const filterEcogesture = (ecogestures: Ecogesture[]) => {
    const filtered = ecogestures
      .filter(ecogesture => Usage[ecogesture.usage] === activeFilter)
      .map(ecogesture => (
        <EcogestureCard
          key={ecogesture.id}
          ecogesture={ecogesture}
          selectionCompleted={selectionViewed === selectionTotal}
        />
      ))
    if (filtered.length > 0) {
      return filtered
    } else {
      return (
        <div className="ec-filter-error">
          <div className="text-20-normal">
            {t('ecogesture.no_ecogesture_filter.text1')}
          </div>
          <div className="text-16-italic">
            {t('ecogesture.no_ecogesture_filter.text2')}
          </div>
        </div>
      )
    }
  }

  const renderEcogestureContent = () => {
    if (list.length > 0) {
      if (activeFilter === Usage[Usage.ALL]) {
        return list.map(ecogesture => (
          <EcogestureCard
            key={ecogesture.id}
            ecogesture={ecogesture}
            selectionCompleted={selectionViewed === selectionTotal}
          />
        ))
      } else {
        return filterEcogesture(list)
      }
    } else if (!displaySelection) {
      return (
        <div className="ec-filter-error">
          <div className="text-20-normal">
            {t('ecogesture.no_ecogesture_filter.text1')}
          </div>
          <div className="text-16-italic">
            {t('ecogesture.no_ecogesture_filter.text2')}
          </div>
        </div>
      )
    }
  }

  return (
    <div className="ecogesture-root">
      <div className="efficiency-button-content">
        {displaySelection ? (
          <div className="selection text-16-normal">
            <span>{t('ecogesture.selection')}</span>
            <span>
              {`(${selectionViewed} ${t(
                'ecogesture.selection_2'
              )} ${selectionTotal})`}
            </span>
            <Button
              aria-label={t('ecogesture.accessibility.button_selection')}
              classes={{
                root: 'btn-highlight',
                label: 'text-16-bold',
              }}
              onClick={() => navigate('/ecogesture-selection')}
            >
              {t('ecogesture.button_selection')}
            </Button>
          </div>
        ) : (
          <div className="filters text-16-normal">
            <div
              className="filter-button"
              onClick={() => toggleDropDown()}
              tabIndex={0}
              onBlur={e => {
                if (e.relatedTarget === null) toggleDropDown()
              }}
            >
              <Button
                classes={{
                  root: 'btn-secondary-negative',
                  label: 'text-14-normal',
                }}
                aria-controls="simple-menu"
                aria-haspopup="true"
                aria-label={t(`ecogesture.MENU_TITLE`)}
                onClick={toggleMenu}
                variant="contained"
              >
                <StyledIcon icon={SortIcon} size={20} />
                <span
                  className={
                    openDropDown ? 'ecogestures opened' : 'ecogestures'
                  }
                >
                  {activeFilter === 'ALL'
                    ? t(`ecogesture.MENU_TITLE`)
                    : t(`ecogesture.${activeFilter}`)}
                </span>
              </Button>
              <Menu
                anchorEl={anchorEl}
                keepMounted
                open={Boolean(anchorEl)}
                onClose={() => closeMenu()}
                PopoverClasses={{
                  paper: 'filter-menu',
                }}
                variant="menu"
                MenuListProps={{ className: 'filter-menu-list' }}
              >
                {Object.values(Usage).map(
                  (usage, key) =>
                    typeof usage !== 'number' && (
                      <MenuItem
                        classes={{
                          root: `${
                            usage === activeFilter ? 'item-active' : ''
                          }`,
                        }}
                        key={key}
                        selected={usage === activeFilter}
                        onClick={() => closeMenu(usage)}
                      >
                        {t(`ecogesture.${usage}`)}
                        {usage === activeFilter && (
                          <ListItemIcon classes={{ root: 'filter-menu-icon' }}>
                            <StyledIcon icon={CheckIcon} size={13} />
                          </ListItemIcon>
                        )}
                      </MenuItem>
                    )
                )}
              </Menu>
            </div>
          </div>
        )}
      </div>
      <div className="ecogesture-content">
        {renderEcogestureContent()}
        {!displaySelection && handleReinitClick && (
          <Button
            onClick={handleReinitClick}
            classes={{
              root: 'btn-secondary-negative',
              label: 'text-16-normal',
            }}
          >
            {t('ecogesture.reinit')}
          </Button>
        )}
      </div>
    </div>
  )
}

export default EcogestureList
