import IconButton from '@material-ui/core/IconButton'
import doingDisabledIcon from 'assets/icons/ico/doing-disabled.svg'
import doingEnabledIcon from 'assets/icons/ico/doing-enabled.svg'
import objectiveDisabledIcon from 'assets/icons/ico/objective-disabled.svg'
import objectiveEnabledIcon from 'assets/icons/ico/objective-enabled.svg'
import defaultIcon from 'assets/icons/visu/ecogesture/default.svg'
import classNames from 'classnames'
import ErrorPage from 'components/CommonKit/ErrorPage/ErrorPage'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import Content from 'components/Content/Content'
import CozyBar from 'components/Header/CozyBar'
import Header from 'components/Header/Header'
import useExploration from 'components/Hooks/useExploration'
import Loader from 'components/Loader/Loader'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import Icon from 'cozy-ui/transpiled/react/Icon'
import { Ecogesture } from 'models'
import React, { useCallback, useEffect, useMemo, useState } from 'react'
import { useSelector } from 'react-redux'
import { Location, useLocation, useNavigate, useParams } from 'react-router-dom'
import EcogestureService from 'services/ecogesture.service'
import { AppStore } from 'store'
import { importIconById } from 'utils/utils'
import EfficiencyRating from './EfficiencyRating'
import './singleEcogesture.scss'

interface EcogestureLocation extends Location {
  state: {
    selectionCompleted: boolean
  }
}

const SingleEcogesture = () => {
  const { t } = useI18n()
  const client = useClient()
  const navigate = useNavigate()
  const location: EcogestureLocation = useLocation()
  const [ecogesture, setEcogesture] = useState<Ecogesture>()
  const [ecogestureIcon, setEcogestureIcon] = useState<string>('')
  const [isMoreDetail, setIsMoreDetail] = useState<boolean>(false)
  const [isDoing, setIsDoing] = useState<boolean>(false)
  const [isObjective, setIsObjective] = useState<boolean>(false)
  const [isLoading, setIsLoading] = useState<boolean>(true)
  const { ecogestureID } = useParams<{ ecogestureID: string }>()
  const selectionCompleted = location?.state?.selectionCompleted

  const ecogestureService = useMemo(
    () => new EcogestureService(client),
    [client]
  )
  const { currentChallenge } = useSelector(
    (state: AppStore) => state.ecolyo.challenge
  )
  const [headerHeight, setHeaderHeight] = useState<number>(0)
  const defineHeaderHeight = (height: number) => {
    setHeaderHeight(height)
  }
  const [, setValidExploration] = useExploration()
  const toggleMoreDetail = () => {
    setIsMoreDetail(prev => !prev)
  }
  const toggleObjective = useCallback(async () => {
    if (ecogesture) {
      const toUpdate: Ecogesture = { ...ecogesture, objective: !isObjective }
      const updatedEcogesture = await ecogestureService.updateEcogesture(
        toUpdate
      )
      if (updatedEcogesture) {
        setIsObjective(prev => !prev)
        setEcogesture(updatedEcogesture)
      }
    }
  }, [ecogesture, ecogestureService, isObjective])

  const toggleDoing = useCallback(async () => {
    if (ecogesture) {
      const toUpdate: Ecogesture = { ...ecogesture, doing: !isDoing }
      const updatedEcogesture = await ecogestureService.updateEcogesture(
        toUpdate
      )
      if (updatedEcogesture) {
        setIsDoing(prev => !prev)
        setEcogesture(updatedEcogesture)
      }
    }
  }, [ecogesture, ecogestureService, isDoing])

  useEffect(() => {
    let subscribed = true
    async function getSingleEcogesture() {
      const data = await ecogestureService.getEcogesturesByIds([
        ecogestureID || '',
      ])
      if (subscribed) {
        if (data?.[0]) {
          setEcogesture(data[0])
          // Prevent case this key doesn't exist in doctype
          setIsObjective(data[0].objective)
          setIsDoing(data[0].doing)
          const icon = await importIconById(data[0].id, 'ecogesture')
          if (subscribed) {
            setEcogestureIcon(icon || defaultIcon)
            if (currentChallenge?.exploration.ecogesture_id === data[0]._id) {
              setValidExploration(currentChallenge.exploration.id)
            }
          }
        }
        setIsLoading(false)
      }
    }
    getSingleEcogesture()

    return () => {
      subscribed = false
    }
  }, [
    client,
    currentChallenge,
    ecogestureID,
    ecogestureService,
    setValidExploration,
  ])

  if (isLoading) {
    return (
      <Content height={headerHeight}>
        <Loader />
      </Content>
    )
  }

  if (ecogesture) {
    return (
      <>
        <CozyBar
          titleKey={'common.title_ecogesture'}
          displayBackArrow={true}
          backFunction={() => navigate('/ecogestures')}
        />
        <Header
          setHeaderHeight={defineHeaderHeight}
          desktopTitleKey={'common.title_ecogesture'}
          displayBackArrow={true}
        />
        <Content height={headerHeight}>
          <div className="single-ecogesture">
            <div className="icon-container">
              {ecogestureIcon && (
                <StyledIcon
                  className="icon-big"
                  icon={ecogestureIcon}
                  size={220}
                />
              )}
            </div>
            <div className="details">
              <div className="text-22 title">{ecogesture.shortName}</div>
              <div className="efficiency">
                <span className="text text-14-normal">
                  {t('ecogesture_modal.efficiency')}
                </span>
                <EfficiencyRating result={Math.round(ecogesture.efficiency)} />
              </div>
            </div>
            <div className="styled-container">
              <div className="long-name text-18-bold">
                {ecogesture.longName}
              </div>
              <div
                className="toggle-text text-15-normal"
                onClick={toggleMoreDetail}
                role="button"
              >
                {isMoreDetail
                  ? t('ecogesture_modal.show_less')
                  : t('ecogesture_modal.show_more')}
              </div>
              <div
                className={classNames('description text-16-normal-150', {
                  ['block']: isMoreDetail === true,
                })}
              >
                {ecogesture.longDescription}
              </div>
            </div>
            {selectionCompleted && (
              <div className="buttons-selection">
                <IconButton
                  aria-label={t('ecogesture.objective')}
                  onClick={toggleObjective}
                  classes={{
                    root: `btn-secondary-negative objective-btn ${
                      isObjective && 'active'
                    }`,
                    label: 'text-15-normal',
                  }}
                >
                  <Icon
                    className="status-icon"
                    icon={
                      isObjective ? objectiveEnabledIcon : objectiveDisabledIcon
                    }
                    size={40}
                  />
                  <span>{t('ecogesture.objective')}</span>
                </IconButton>
                <IconButton
                  aria-label={t('ecogesture.doing')}
                  onClick={toggleDoing}
                  classes={{
                    root: `btn-secondary-negative doing-btn ${
                      isDoing && 'active'
                    }`,
                    label: 'text-15-normal',
                  }}
                >
                  <Icon
                    className="status-icon"
                    icon={isDoing ? doingEnabledIcon : doingDisabledIcon}
                    size={40}
                  />
                  <span>{t('ecogesture.doing')}</span>
                </IconButton>
              </div>
            )}
          </div>
        </Content>
      </>
    )
  }

  return (
    <ErrorPage
      text={t('error_page.no_ecogesture')}
      returnPage={'ecogestures'}
    />
  )
}

export default SingleEcogesture
