import Button from '@material-ui/core/Button'
import * as Sentry from '@sentry/react'
import finishIcon from 'assets/icons/visu/profileType/finish.svg'
import StyledIcon from 'components/CommonKit/Icon/StyledIcon'
import useExploration from 'components/Hooks/useExploration'
import 'components/ProfileType/profileTypeFinished.scss'
import { Client, useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { PROFILETYPE_DOCTYPE } from 'doctypes'
import { UsageEventType } from 'enum/usageEvent.enum'
import { UserExplorationID } from 'enum/userExploration.enum'
import { DateTime } from 'luxon'
import { TimePeriod } from 'models'
import { ProfileType } from 'models/profileType.model'
import React, { Dispatch, useEffect, useState } from 'react'
import { useDispatch, useSelector } from 'react-redux'
import { useLocation, useNavigate } from 'react-router-dom'
import ProfileTypeService from 'services/profileType.service'
import ProfileTypeEntityService from 'services/profileTypeEntity.service'
import UsageEventService from 'services/usageEvent.service'
import { AppActionsTypes, AppStore } from 'store'
import { updateProfile } from 'store/profile/profile.actions'
import { setProfileType } from 'store/profileType/profileType.slice'

const ProfileTypeFinished = ({ profileType }: { profileType: ProfileType }) => {
  const { t } = useI18n()
  const location = useLocation()
  const dispatch = useDispatch<Dispatch<AppActionsTypes>>()
  const navigate = useNavigate()
  const client = useClient()
  const {
    challenge: { currentChallenge },
    profile,
  } = useSelector((state: AppStore) => state.ecolyo)
  const handleClick = () => {
    if (location?.pathname === '/ecogesture-form') {
      navigate('/ecogesture-selection')
    } else {
      navigate(-1)
    }
  }

  const [isSaved, setIsSaved] = useState<boolean>(false)
  const [, setValidExploration] = useExploration()

  useEffect(() => {
    async function checkForExistingProfileType() {
      const consistentProfileType =
        ProfileTypeService.checkConsistency(profileType)
      const chosenPeriod: TimePeriod = {
        startDate: profileType.updateDate.setZone('utc', {
          keepLocalTime: true,
        }),
        endDate: DateTime.local().setZone('utc', {
          keepLocalTime: true,
        }),
      }
      const profileTypeEntityService = new ProfileTypeEntityService(client)
      const myProfileTypes: ProfileType[] | null =
        await profileTypeEntityService.getAllProfileTypes(chosenPeriod)
      if (myProfileTypes !== null) {
        const destroyPT = await profileTypeEntityService.deleteProfileTypes(
          myProfileTypes
        )
        if (destroyPT) {
          await createNewProfileType(client, consistentProfileType)
          setIsSaved(true)
          dispatch(
            updateProfile({
              isProfileEcogestureCompleted: true,
              isProfileTypeCompleted: true,
            })
          )
        } else {
          console.log('ERROR')
          Sentry.captureException('error in profileTypeFinished')
        }
      } else {
        await createNewProfileType(client, consistentProfileType)
        setIsSaved(true)
        dispatch(
          updateProfile({
            isProfileEcogestureCompleted: true,
            isProfileTypeCompleted: true,
          })
        )
      }
    }

    async function createNewProfileType(
      client: Client,
      consistentProfileType: ProfileType
    ) {
      const { data: newProfileType } = await client.create(
        PROFILETYPE_DOCTYPE,
        consistentProfileType
      )

      if (newProfileType) {
        dispatch(setProfileType(newProfileType))
      }
    }

    if (!isSaved) {
      checkForExistingProfileType()
      if (
        currentChallenge &&
        currentChallenge.exploration.id === UserExplorationID.EXPLORATION001
      ) {
        setValidExploration(currentChallenge.exploration.id)
      }
      UsageEventService.addEvent(client, {
        type: UsageEventType.PROFILE_SET_EVENT,
      })
    }
  }, [
    dispatch,
    profileType,
    isSaved,
    profile.isProfileTypeCompleted,
    currentChallenge,
    setValidExploration,
    client,
  ])

  return (
    <div className={'profile-type-finished-card'}>
      <StyledIcon
        className={'profile-type-icon'}
        icon={finishIcon}
        size={120}
      />
      <div className={'profile-type-finished-label text-28-normal-uppercase'}>
        {t('profile_type.finished.title')}
      </div>
      <div className={'profile-type-finished-description text-18-normal'}>
        <div>{t('profile_type.finished.label1')}</div>
        <div>{t('profile_type.finished.label2')}</div>
      </div>
      <Button
        aria-label={t('profile_type.accessibility.button_validate')}
        onClick={handleClick}
        className={'profile-type-finished-button'}
        classes={{
          root: 'btn-primary-positive',
          label: 'text-16-normal',
        }}
      >
        {t('profile_type.finished.button_validate')}
      </Button>
    </div>
  )
}

export default ProfileTypeFinished
