import { Button } from '@material-ui/core'
import TimeStepSelector from 'components/TimeStepSelector/TimeStepSelector'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { mount } from 'enzyme'
import toJson from 'enzyme-to-json'
import { DateTime } from 'luxon'
import React from 'react'
import * as reactRedux from 'react-redux'
import { Provider } from 'react-redux'
import UsageEventService from 'services/usageEvent.service'
import * as chartActions from 'store/chart/chart.slice'
import { createMockEcolyoStore } from '../../../tests/__mocks__/store'
import { waitForComponentToPaint } from '../../../tests/__mocks__/testUtils'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})
jest.mock('services/usageEvent.service')
const mockAddEvent = jest.fn()
UsageEventService.addEvent = mockAddEvent

const useSelectorSpy = jest.spyOn(reactRedux, 'useSelector')
const useDispatchSpy = jest.spyOn(reactRedux, 'useDispatch')
const setCurrentTimeStepSpy = jest.spyOn(chartActions, 'setCurrentTimeStep')
const setCurrentIndexSpy = jest.spyOn(chartActions, 'setCurrentIndex')
const setSelectedDateSpy = jest.spyOn(chartActions, 'setSelectedDate')

describe('TimeStepSelector component', () => {
  const store = createMockEcolyoStore()
  beforeEach(() => {
    store.clearActions()
    useSelectorSpy.mockClear()
    useDispatchSpy.mockClear()
    setCurrentTimeStepSpy.mockClear()
  })

  it('should render component properly with 4 timesteps', async () => {
    useSelectorSpy.mockReturnValue({
      currentTimeStep: TimeStep.DAY,
      selectedDate: DateTime.local().setZone('utc', {
        keepLocalTime: true,
      }),
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.WATER} />
      </Provider>
    )
    expect(wrapper.find('.circle').at(3).exists()).toBeTruthy()
    expect(wrapper.find('.circle').at(4).exists()).toBeFalsy()
    await waitForComponentToPaint(wrapper)
    expect(toJson(wrapper)).toMatchSnapshot()
  })

  it('should render component properly with 5 timesteps', () => {
    useSelectorSpy.mockReturnValue({
      currentTimeStep: TimeStep.DAY,
      selectedDate: DateTime.local().setZone('utc', {
        keepLocalTime: true,
      }),
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.ELECTRICITY} />
      </Provider>
    )
    expect(wrapper.find('.circle').at(4).exists()).toBeTruthy()
    expect(wrapper.find('.circle').at(5).exists()).toBeFalsy()
  })

  it('should define next TimeStep and dispatch it', () => {
    useSelectorSpy.mockReturnValue({
      currentTimeStep: TimeStep.YEAR,
      selectedDate: DateTime.local().setZone('utc', {
        keepLocalTime: true,
      }),
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.WATER} />
      </Provider>
    )
    wrapper.find('#day').first().simulate('click')
    expect(setCurrentTimeStepSpy).toBeCalledTimes(1)
    expect(setCurrentTimeStepSpy).toHaveBeenCalledWith(TimeStep.DAY)
    expect(setCurrentIndexSpy).toBeCalledTimes(1)
  })
  it('should go to todays day with timestep week', () => {
    useSelectorSpy.mockReturnValue({
      currentTimeStep: TimeStep.YEAR,
      selectedDate: DateTime.local().setZone('utc', {
        keepLocalTime: true,
      }),
    })
    const wrapper = mount(
      <Provider store={store}>
        <TimeStepSelector fluidType={FluidType.WATER} />
      </Provider>
    )
    wrapper.find(Button).first().simulate('click')
    expect(setCurrentTimeStepSpy).toBeCalledTimes(1)
    expect(setCurrentTimeStepSpy).toHaveBeenCalledWith(TimeStep.WEEK)
    expect(setCurrentIndexSpy).toBeCalledTimes(2)
    expect(setSelectedDateSpy).toBeCalledTimes(1)
  })
})
