import * as Sentry from '@sentry/react'
import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import logger from 'cozy-logger'
import ecogestureData from 'db/ecogestureData.json'
import { ECOGESTURE_DOCTYPE } from 'doctypes'
import { Season, Usage } from 'enum/ecogesture.enum'
import { FluidType } from 'enum/fluid.enum'
import { IndividualOrCollective, WarmingType } from 'enum/profileType.enum'
import { orderBy } from 'lodash'
import { Ecogesture } from 'models'
import { ProfileEcogesture } from 'models/profileEcogesture.model'
import { logDuration } from 'utils/duration'
import { hashFile } from 'utils/hash'
import logApp from 'utils/logger'

const logStack = logger.namespace('ecogestureService')

export default class EcogestureService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  /**
   * - Load ecogestures if they exists.
   * - If not create them.
   * - If hash mismatch, update ecogestures.
   */
  public async initEcogesture(hash: string): Promise<{
    ecogestureHash: string
    ecogestureList: Ecogesture[]
  }> {
    const startTime = performance.now()
    const hashEcogestureType = hashFile(ecogestureData)
    const ecogestures = await this.getAllEcogestures(undefined, true)

    if (!ecogestures || ecogestures?.length === 0) {
      // Populate data if none ecogesture exists
      try {
        for (const ecogesture of ecogestureData) {
          await this._client.create(ECOGESTURE_DOCTYPE, ecogesture)
        }
        // Check of created document based on count
        const ecogestures = await this.getAllEcogestures()
        if (!ecogestures || ecogestures?.length !== ecogestureData.length) {
          throw new Error(
            'initEcogesture: Created ecogesture type entities does not match'
          )
        }
        logDuration('[Initialization] Ecogesture list created', startTime)
        return {
          ecogestureHash: hashEcogestureType,
          ecogestureList: ecogestures,
        }
      } catch (error) {
        const errorMessage = `Initialization error - initEcogesture: ${JSON.stringify(
          error
        )}`
        logStack('error', errorMessage)
        logApp.error(errorMessage)
        Sentry.captureException(errorMessage)
        throw error
      }
    }
    // Update if the hash is not the same as the one from profile
    if (hash !== hashEcogestureType) {
      // Update the doctype
      try {
        // Deletion of all documents
        await this.deleteAllEcogestures()
        // Population with the data
        for (const [index, ecogesture] of ecogestureData.entries()) {
          const updateEcogesture = ecogestures[index]
            ? {
                ...ecogesture,
                objective: ecogestures[index].objective,
                doing: ecogestures[index].doing,
                viewedInSelection: ecogestures[index].viewedInSelection,
              }
            : ecogesture
          await this._client.create(ECOGESTURE_DOCTYPE, updateEcogesture)
        }
        // Check of created document based on count
        const checkCount = await this.getAllEcogestures()
        if (!checkCount || checkCount?.length !== ecogestureData.length) {
          throw new Error(
            'initEcogesture: Created ecogesture type entities does not match'
          )
        }
        logDuration('[Initialization] Ecogesture updated', startTime)
        return {
          ecogestureHash: hashEcogestureType,
          ecogestureList: checkCount,
        }
      } catch (error) {
        const errorMessage = `Initialization error - initEcogesture: ${JSON.stringify(
          error
        )}`
        logStack('error', errorMessage)
        logApp.error(errorMessage)
        Sentry.captureException(errorMessage)
        throw error
      }
    } else {
      // Doctype already up to date
      logDuration('[Initialization] Ecogesture already up-to-date', startTime)
      return {
        ecogestureHash: hashEcogestureType,
        ecogestureList: ecogestures,
      }
    }
  }

  // TODO add default params
  public async getAllEcogestures(
    seasonFilter?: Season,
    orderByID?: boolean
  ): Promise<Ecogesture[]> {
    let query: QueryDefinition = Q(ECOGESTURE_DOCTYPE)
    if (seasonFilter && seasonFilter !== Season.NONE) {
      query = query
        .where({ season: { $ne: seasonFilter } })
        .indexFields(['season'])
        .sortBy([{ season: 'desc' }])
    } else if (orderByID) {
      query = query
        .where({})
        .indexFields(['_id'])
        .sortBy([{ _id: 'asc' }])
    } else {
      query = query
        .where({})
        .indexFields(['season'])
        .sortBy([{ season: 'desc' }])
    }

    const { data: ecogestures }: QueryResult<Ecogesture[]> =
      await this._client.query(query)

    if (seasonFilter && seasonFilter !== Season.NONE) {
      const { data: ecogesturesWithSeason }: QueryResult<Ecogesture[]> =
        await this._client.query(
          Q(ECOGESTURE_DOCTYPE)
            .where({ season: { $eq: seasonFilter } })
            .indexFields(['season'])
            .sortBy([{ season: 'asc' }])
        )
      return [...ecogesturesWithSeason, ...ecogestures]
    }
    return ecogestures
  }

  /**
   * @param {string} ids - ecogestures ids
   * @returns {Promise<Ecogesture[]>}
   */
  public async getEcogesturesByIds(ids: string[]): Promise<Ecogesture[]> {
    const query: QueryDefinition = Q(ECOGESTURE_DOCTYPE).getByIds(ids)
    const { data: ecogestures }: QueryResult<Ecogesture[]> =
      await this._client.query(query)
    return ecogestures
  }

  public async deleteAllEcogestures(): Promise<boolean> {
    const ecogestures = await this.getAllEcogestures()
    try {
      for (const ecogesture of ecogestures) {
        await this._client.destroy(ecogesture)
      }
      return true
    } catch (error) {
      const errorMessage = `Error deleteAllEcogestures: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw error
    }
  }

  public async reinitAllEcogestures(): Promise<boolean> {
    const ecogestures = await this.getAllEcogestures()
    try {
      for (const ecogesture of ecogestures) {
        await this.updateEcogesture({
          ...ecogesture,
          objective: false,
          doing: false,
          viewedInSelection: false,
        })
      }
      return true
    } catch (error) {
      const errorMessage = `Error reinitAllEcogestures: ${JSON.stringify(
        error
      )}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      throw error
    }
  }

  /**
   * Removes ecogestures from the list that doesn't fit with user's usages
   * @param {Ecogesture[]} ecogestureList
   * @param {ProfileEcogesture} profileEcogesture
   * @returns {Ecogesture[]}
   */
  public filterByUsage(
    ecogestureList: Ecogesture[],
    profileEcogesture: ProfileEcogesture
  ): Ecogesture[] {
    const filteredByUsage: Ecogesture[] = ecogestureList.filter(ecogesture => {
      switch (ecogesture.usage) {
        case Usage.HEATING:
          if (
            ecogesture.fluidTypes.includes(FluidType.ELECTRICITY) &&
            profileEcogesture.heating === IndividualOrCollective.INDIVIDUAL &&
            profileEcogesture.warmingFluid === WarmingType.ELECTRICITY
          ) {
            return true
          } else if (
            ecogesture.fluidTypes.includes(FluidType.GAS) &&
            profileEcogesture.heating === IndividualOrCollective.INDIVIDUAL &&
            profileEcogesture.warmingFluid === WarmingType.GAS
          ) {
            return true
          } else {
            return false
          }
        case Usage.ECS:
          if (
            profileEcogesture.hotWater === IndividualOrCollective.INDIVIDUAL
          ) {
            return true
          } else {
            return false
          }
        default:
          return true
      }
    })
    return filteredByUsage
  }

  /**
   * Removes ecogesture from the list that depends on equipment the user hasn't
   * @param {Ecogesture[]} ecogestureList
   * @param {ProfileEcogesture} profileEcogesture
   * @returns {Ecogesture[]}
   */
  public filterByEquipment(
    ecogestureList: Ecogesture[],
    profileEcogesture: ProfileEcogesture
  ): Ecogesture[] {
    for (const ecogesture of ecogestureList) {
      if (ecogesture.equipment === true) {
        for (const equipmentType of ecogesture.equipmentType) {
          if (!profileEcogesture.equipments.includes(equipmentType)) {
            const index = ecogestureList.indexOf(ecogesture)
            ecogestureList.splice(index, 1)
          }
        }
      }
    }
    return ecogestureList
  }

  /**
   * Return a filtered list according to ecogesture profile, the list is sorted by low difficulty and high efficiency
   * @param {ProfileEcogesture} profileEcogesture
   * @returns {Ecogesture[]}
   */
  public async getEcogestureListByProfile(
    profileEcogesture: ProfileEcogesture
  ): Promise<Ecogesture[]> {
    const ecogestureList: Ecogesture[] = await this.getAllEcogestures()
    const filteredByUsage: Ecogesture[] = this.filterByUsage(
      ecogestureList,
      profileEcogesture
    )
    const filteredByEquipment: Ecogesture[] = this.filterByEquipment(
      filteredByUsage,
      profileEcogesture
    )
    const filteredFlaggedEcogesture: Ecogesture[] = filteredByEquipment.filter(
      (ecogesture: Ecogesture) =>
        (ecogesture.objective === false &&
          ecogesture.doing === false &&
          ecogesture.viewedInSelection === false) ||
        ecogesture.viewedInSelection === true
    )
    const sortedByDifficultyAndEfficiency: Ecogesture[] = orderBy(
      filteredFlaggedEcogesture,
      [
        ecogesture => {
          return ecogesture.difficulty
        },
        ecogesture => {
          return ecogesture.efficiency
        },
      ],
      ['asc', 'desc']
    )
    return sortedByDifficultyAndEfficiency
  }

  /**
   * Update one ecogesture
   * @param {Ecogesture} ecogesture - Ecogesture to save
   * @returns {Ecogesture} Updated Ecogesture
   */
  public async updateEcogesture(ecogesture: Ecogesture): Promise<Ecogesture> {
    const { data: updatedEcogesture }: QueryResult<Ecogesture> =
      await this._client.save(ecogesture)
    return updatedEcogesture
  }
}
